/****************************************************************************
 * Projekt - HMI-Unit Demo
 * (c) 2021 softwareentwicklung-als-prozess.de
 * Dokumentation s. Blog
 * Funktionalitaet:
 * Alle Elemente einer Mensch Maschine Schnittstelle auf einem Board als 
 * unabhaengige Einheit auf einem Board. 
 * Gesteuert ueber eine serielle Schnittstelle von einem anderen Controller.
 ***************************************************************************/
/* --- Compiler Includes ---------- */
#include <arduino.h>

// defines
// Index fuer die Kommandos
#define BLUELEDON  6
#define BLUELEDOFF 8
#define REDLEDON 9
#define REDLEDOFF 11
#define GREENLEDON 7
#define GREENLEDOFF 10

#define NOKEY 0
#define BKEY 1
#define AKEY 2
#define LKEY 3
#define DKEY 4
#define UKEY 5
#define RKEY 6
// Standardwrtezeit in ms
#define MINWAIT 100
#define SEKUNDE 1000

// HMI Unit Kommandos im Flash memory
const char txt0[]  PROGMEM = "dsp clean\n";
const char txt1[]  PROGMEM = "dsp size 2\n";
const char txt2[]  PROGMEM = "dsp curs 12 16\n";
const char txt3[]  PROGMEM = "dsp text SEaP:\n";
const char txt4[]  PROGMEM = "dsp line 12 14 66 14\n";
const char txt5[]  PROGMEM = "dsp line 12 31 66 31\n";
const char txt6[]  PROGMEM = "led blau an\n";
const char txt7[]  PROGMEM = "led gruen an\n";
const char txt8[]  PROGMEM = "led blau aus\n";
const char txt9[]  PROGMEM = "led rot an\n";
const char txt10[] PROGMEM = "led gruen aus\n";
const char txt11[] PROGMEM = "led rot aus\n";
const char txt12[] PROGMEM = "led back an\n";
const char txt13[] PROGMEM = "led back aus\n";
const char txt14[] PROGMEM = "buz an\n";
const char txt15[] PROGMEM = "buz aus\n";
const char txt16[] PROGMEM = "key push\n";
const char txt17[] PROGMEM = "dsp size 1\n"; // key abfrage anzeigen
const char txt18[] PROGMEM = "dsp curs 18 00\n"; // Tastenausgabe
const char txt19[] PROGMEM = "dsp curs 18 38\n"; // Temperaturausgabe
// Ausgabezeile fuer letzte gedrueckte Taste
const char key0[]  PROGMEM = "dsp text ------\n";
const char key1[]  PROGMEM = "dsp text B-----\n";
const char key2[]  PROGMEM = "dsp text -A----\n";
const char key3[]  PROGMEM = "dsp text --L---\n"; // left
const char key4[]  PROGMEM = "dsp text ---D--\n"; // down
const char key5[]  PROGMEM = "dsp text ----U-\n"; // up
const char key6[]  PROGMEM = "dsp text -----R\n"; // right

const char *const cmdtbl[] PROGMEM = {txt0, txt1, txt2, txt3, txt4, txt5, txt6, txt7, txt8, txt9, txt10, txt11, txt12, txt13, txt14, txt15, txt16, txt17, txt18, txt19};
const char *const keytbl[] PROGMEM = {key0, key1, key2, key3, key4, key5, key6};

// vars
static char sndbuf[25];
String inputString = "";
unsigned long starttime  = 0ul;
int  toggleled = BLUELEDON;
char key = (char)0;
bool stringComplete = false;
int  pin = A0;

// interne prototypes
static void  ser_Init(void);
static void  ser_CheckInput(void);
static void  hmi_OutputMask(void);
static void  hmi_DisplayMask(void);
static void  hmi_ClearSndBuf(void);
static float tmp_TempRead(void);
static void  ser_SendKeyLine(int keyinx);
static void  ser_SendCommand(int cmdinx);

// funcs
void setup() 
{
  ser_Init();           // serielle Schnittstelle initialisieren
  pinMode(pin, INPUT);  // Analog-eingang LM35 Temperatursensor ohne Pullup
  hmi_OutputMask();     // Led's und Buzzer ansteuern
  hmi_DisplayMask();    // Display ansteuern
  starttime = millis(); // beginn der zeitmessung
}

void loop() 
{
  ser_CheckInput();             // Meldungen von HMI Unit auswerten
  if ((millis() - starttime) > SEKUNDE)  // Eine Sekunde ist abgelaufen
  {
    starttime = millis();       // beginn der zeitmessung
    ser_SendCommand(toggleled); // toggle An und AUS
    if (toggleled == BLUELEDON)
    {
      toggleled = BLUELEDOFF;   // AUS
    }
    else
    {
      toggleled = BLUELEDON;    // AN
    }
  }
}

// serielle Schnittstelle initialisieren
static void ser_Init(void)
{
  Serial1.begin(115200, SERIAL_8N1);
  inputString.reserve(50);
}

// sendungen vom HMI Unit auswerten
static void ser_CheckInput(void)
{
  if (stringComplete == true)
  {
    if(inputString.compareTo("Ok\n") == 0)
    {
    }
    if(inputString.compareTo("B\n") == 0)
    {
      key = 'B';
      hmi_DisplayMask();
    }
    if(inputString.compareTo("A\n") == 0)
    {
      key = 'A';
      hmi_DisplayMask();
    }
    if(inputString.compareTo("L\n") == 0)
    {
      key = 'L';
      hmi_DisplayMask();
    }
    if(inputString.compareTo("D\n") == 0)
    {
      key = 'D';
      hmi_DisplayMask();
    }
    if(inputString.compareTo("U\n") == 0)
    {
      key = 'U';
      hmi_DisplayMask();
    }
    if(inputString.compareTo("R\n") == 0)
    {
      key = 'R';
      hmi_DisplayMask();
    }
    inputString = "";
    stringComplete = false;
  }
}

// Sendepuffer loeschen
static void hmi_ClearSndBuf(void)
{
  int j = 0;
  
  for(j = 0; j < sizeof(sndbuf); j++)
  {
    sndbuf[j] = 0;
  }  
}

// LED und Buzzer Kommandos ausfuehren
static void hmi_OutputMask(void) // nur LED's und Buzzer
{
  int i;
  
  // Ausgabe LED und Buzzer Kommandos
  for(i = 6; i < 16; i++)
  {
    ser_SendCommand(i);
    if( (i == 7) || (i == 9) || (i == 12) || (i == 14))
    {
      delay(SEKUNDE);
    }
    else
    {
      delay(MINWAIT);
    }
  }
}

// Displayausgaben ausfuehren
static void hmi_DisplayMask(void) // nur Display
{
  int i;
  String Temp = "dsp text T:";

  // Grundmaske ausgeben
  for(i = 0; i < 6; i++)
  {
    ser_SendCommand(i);
    delay(MINWAIT);
  }
  for(i = 16; i < 19; i++)
  {
    ser_SendCommand(i);
    delay(MINWAIT);
  }
  // Zeile mit Tastenkennung ausgeben
  switch(key)
  {
    case 'B':
      ser_SendKeyLine(BKEY);
      break;  
    case 'A':
      ser_SendKeyLine(AKEY);
      break;  
    case 'L':
      ser_SendKeyLine(LKEY);
      break;  
    case 'D':
      ser_SendKeyLine(DKEY);
      break;  
    case 'U':
      ser_SendKeyLine(UKEY);
      break;  
    case 'R':
      ser_SendKeyLine(RKEY);
      break;  
    default:
      ser_SendKeyLine(NOKEY);
      break;
  }
  delay(MINWAIT);
  // Temperaturausgabe
  ser_SendCommand(19);
  delay(MINWAIT);
  Temp = Temp + String(tmp_TempRead(),1) + "'C\n";
  Serial1.print(Temp);
  delay(MINWAIT);
  key = (char)0;
  delay(MINWAIT);
}

// Callback fuer das empfangen von Zeichen auf der seriellen Schnittstelle
void serialEvent1(void)
{
  char inChar;
  
  while (Serial1.available()) 
  {
    inChar = (char)Serial1.read();
    inputString += inChar;
    if (inChar == '\n') 
    {
      stringComplete = true;
    }
  }
}

// Temperatur vom LM35 Sensor einlesen
static float tmp_TempRead(void)
{
  return((float(analogRead(pin))*5/(1023))/0.01);
}

// Kommando fuer die Tastenzeile senden
static void ser_SendKeyLine(int keyinx)
{
  hmi_ClearSndBuf();
  strcpy_P(sndbuf,(char *)pgm_read_word(&(keytbl[keyinx])));
  Serial1.write(sndbuf);
}

// Kommando fuer LED's, Buzzer und Display senden
static void ser_SendCommand(int cmdinx)
{
  hmi_ClearSndBuf();
  strcpy_P(sndbuf,(char *)pgm_read_word(&(cmdtbl[cmdinx])));
  Serial1.write(sndbuf);  
}
